package com.kakao.sdk.auth.network

import android.content.Context
import com.kakao.sdk.auth.*
import com.kakao.sdk.auth.model.AccessTokenResponse
import com.kakao.sdk.common.*
import com.kakao.sdk.common.model.ApiError
import com.kakao.sdk.common.model.ApiErrorCause
import io.reactivex.*
import com.kakao.sdk.common.model.ApplicationInfo
import com.kakao.sdk.common.model.ContextInfo
import io.reactivex.schedulers.Schedulers
import org.reactivestreams.Publisher

/**
 * @author kevin.kang. Created on 2018. 5. 2..
 */
class TokenBasedApiInterceptor(
    private val authApiClient: RxAuthApiClient = AuthApiClient.rx,
    private val accessTokenRepo: AccessTokenRepo = AccessTokenRepo.instance,
    private val appInfo: ApplicationInfo = KakaoSdk.applicationContextInfo,
    private val contextInfo: ContextInfo = KakaoSdk.applicationContextInfo
) {
    fun <T> handleApiError(): SingleTransformer<T, T> =
        SingleTransformer { it ->
            it.retryWhen { refreshAccessToken(it) }
                .doOnError {
                    if (it is ApiError && it.reason == ApiErrorCause.InvalidToken)
                        accessTokenRepo.clearCache()
                }
        }


    fun handleCompletableError(): CompletableTransformer =
        CompletableTransformer {
            it.retryWhen { refreshAccessToken(it) }
                .doOnError {
                    if (it is ApiError && it.reason == ApiErrorCause.InvalidToken)
                        accessTokenRepo.clearCache()
                }
        }

    fun handleInvalidScope(
        context: Context,
        flowable: Flowable<Throwable>
    ): Publisher<AccessTokenResponse> =
        flowable.flatMap {
            if (it is ApiError && it.reason == ApiErrorCause.InvalidScope) {
                return@flatMap AuthCodeClient.rx.authorizeWithNewScopes(
                    context,
                    it.response.requiredScopes!!
                ).observeOn(Schedulers.io())
                    .flatMap { authApiClient.issueAccessToken(it) }.toFlowable()
            }
            throw it
        }

    @JvmSynthetic
    internal fun refreshAccessToken(throwableFlowable: Flowable<Throwable>): Publisher<AccessTokenResponse> =
        throwableFlowable.flatMap {
            val token = accessTokenRepo.fromCache()
            if (token.refreshToken == null || it !is ApiError || it.reason != ApiErrorCause.InvalidToken) {
                throw it
            }
            return@flatMap authApiClient.refreshAccessToken(
                token.refreshToken!!,
                appInfo.clientId,
                appInfo.approvalType,
                contextInfo.signingKeyHash
            ).toFlowable()
        }

    companion object {
        val instance by lazy {
            TokenBasedApiInterceptor()
        }
    }
}