@file:JvmName("StoryApiClientKt")

package com.kakao.sdk.story

import com.kakao.sdk.auth.network.TokenBasedApiInterceptor
import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.ApiFactory
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.network.RxApiInterceptor
import com.kakao.sdk.story.model.*
import io.reactivex.Completable
import io.reactivex.Single
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody.Companion.asRequestBody
import java.io.File

/**
 * @author kevin.kang. Created on 2018. 3. 20..
 */
class RxStoryApiClient(
    private val api: RxStoryApi = ApiFactory.kapiWithOAuth.create(RxStoryApi::class.java),
    private val apiInterceptor: TokenBasedApiInterceptor = TokenBasedApiInterceptor.instance
) {
    fun isStoryUser(): Single<StoryUserResult> {
        return api.isStoryUser()
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    @JvmOverloads
    fun profile(secureResource: Boolean? = true): Single<StoryProfile> {
        return api.profile(secureResource)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    fun story(id: String): Single<Story> {
        return api.story(id)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    @JvmOverloads
    fun stories(lastId: String? = null): Single<List<Story>> {
        return api.stories(lastId)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    @JvmOverloads
    fun postNote(
        content: String,
        permission: Story.Permission = Story.Permission.PUBLIC,
        enableShare: Boolean = true,
        androidExecParams: Map<String, String>? = null,
        iosExecParams: Map<String, String>? = null,
        androidMarketParams: Map<String, String>? = null,
        iosMarketParams: Map<String, String>? = null
    ): Single<StoryPostResult> {
        return api.postNote(
            content, permission, enableShare, androidExecParams, iosExecParams, androidMarketParams,
            iosMarketParams
        ).compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    @JvmOverloads
    fun postLink(
        linkInfo: LinkInfo,
        content: String,
        permission: Story.Permission = Story.Permission.PUBLIC,
        enableShare: Boolean = true,
        androidExecParams: Map<String, String>? = null,
        iosExecParams: Map<String, String>? = null,
        androidMarketParams: Map<String, String>? = null,
        iosMarketParams: Map<String, String>? = null
    ): Single<StoryPostResult> {

        return api.postLink(
            linkInfo, content, permission, enableShare, androidExecParams, iosExecParams,
            androidMarketParams, iosMarketParams
        ).compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    @JvmOverloads
    fun postPhoto(
        images: List<String>,
        content: String,
        permission: Story.Permission = Story.Permission.PUBLIC,
        enableShare: Boolean = true,
        androidExecParams: Map<String, String>? = null,
        iosExecParams: Map<String, String>? = null,
        androidMarketParams: Map<String, String>? = null,
        iosMarketParams: Map<String, String>? = null
    ): Single<StoryPostResult> {
        return api.postPhoto(
            KakaoJson.toJson(images), content, permission, enableShare,
            androidExecParams, iosExecParams, androidMarketParams, iosMarketParams
        ).compose(RxApiInterceptor.handleApiError())
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    fun deleteStory(id: String): Completable {
        return api.deleteStory(id)
            .compose(RxApiInterceptor.handleCompletableError())
            .compose(apiInterceptor.handleCompletableError())
    }

    fun linkInfo(url: String): Single<LinkInfo> {
        return api.linkInfo(url)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    fun upload(images: List<File>): Single<List<String>> {
        return Single.just(images.map {
            Pair(
                it.name,
                it.asRequestBody("image/*".toMediaType())
            )
        }
            .mapIndexed { index, pair ->
                MultipartBody.Part.createFormData(
                    "${Constants.FILE}_$index",
                    pair.first,
                    pair.second
                )
            })
            .flatMap { api.upload(it) }
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())
    }

    companion object {
        @JvmStatic
        val instance by lazy { StoryApiClient.rx }
    }
}

val StoryApiClient.Companion.rx by lazy { RxStoryApiClient() }